"""OMOP CDM v5.3.1 vocabulary tables."""

import datetime
import decimal
from typing import Optional

from sqlalchemy import Date, DateTime, ForeignKey, Integer, Numeric, String
from sqlalchemy.ext.declarative import declared_attr
from sqlalchemy.orm import Mapped, mapped_column, relationship

from omop_cdm.constants import (
    CDM_SCHEMA,
    FK_CONCEPT_CLASS_ID,
    FK_CONCEPT_ID,
    FK_DOMAIN_ID,
    FK_VOCABULARY_ID,
    VOCAB_SCHEMA,
)
from omop_cdm.util import get_current_time_utc


class BaseConceptCdm531:
    __tablename__ = "concept"
    __table_args__ = {"schema": VOCAB_SCHEMA}

    concept_id: Mapped[int] = mapped_column(Integer, primary_key=True, sort_order=100)
    concept_name: Mapped[str] = mapped_column(String(255), sort_order=200)
    domain_id: Mapped[str] = mapped_column(ForeignKey(FK_DOMAIN_ID), index=True, sort_order=300)
    vocabulary_id: Mapped[str] = mapped_column(ForeignKey(FK_VOCABULARY_ID), index=True, sort_order=400)
    concept_class_id: Mapped[str] = mapped_column(ForeignKey(FK_CONCEPT_CLASS_ID), index=True, sort_order=500)
    standard_concept: Mapped[Optional[str]] = mapped_column(String(1), sort_order=600)
    concept_code: Mapped[str] = mapped_column(String(50), index=True, sort_order=700)
    valid_start_date: Mapped[datetime.date] = mapped_column(Date, sort_order=800)
    valid_end_date: Mapped[datetime.date] = mapped_column(Date, sort_order=900)
    invalid_reason: Mapped[Optional[str]] = mapped_column(String(1), sort_order=1000)

    @declared_attr
    def concept_class(cls) -> Mapped["ConceptClass"]:
        return relationship("ConceptClass", foreign_keys="Concept.concept_class_id")

    @declared_attr
    def domain(cls) -> Mapped["Domain"]:
        return relationship("Domain", foreign_keys="Concept.domain_id")

    @declared_attr
    def vocabulary(cls) -> Mapped["Vocabulary"]:
        return relationship("Vocabulary", foreign_keys="Concept.vocabulary_id")


class BaseConceptAncestorCdm531:
    __tablename__ = "concept_ancestor"
    __table_args__ = {"schema": VOCAB_SCHEMA}

    ancestor_concept_id: Mapped[int] = mapped_column(
        ForeignKey(FK_CONCEPT_ID), primary_key=True, index=True, sort_order=100
    )
    descendant_concept_id: Mapped[int] = mapped_column(
        ForeignKey(FK_CONCEPT_ID), primary_key=True, index=True, sort_order=200
    )
    min_levels_of_separation: Mapped[int] = mapped_column(Integer, sort_order=300)
    max_levels_of_separation: Mapped[int] = mapped_column(Integer, sort_order=400)

    @declared_attr
    def ancestor_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="ConceptAncestor.ancestor_concept_id")

    @declared_attr
    def descendant_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="ConceptAncestor.descendant_concept_id")


class BaseConceptClassCdm531:
    __tablename__ = "concept_class"
    __table_args__ = {"schema": VOCAB_SCHEMA}

    concept_class_id: Mapped[str] = mapped_column(String(20), primary_key=True, sort_order=100)
    concept_class_name: Mapped[str] = mapped_column(String(255), sort_order=200)
    concept_class_concept_id: Mapped[int] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=300)

    @declared_attr
    def concept_class_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="ConceptClass.concept_class_concept_id")


class BaseConceptRelationshipCdm531:
    __tablename__ = "concept_relationship"
    __table_args__ = {"schema": VOCAB_SCHEMA}

    concept_id_1: Mapped[int] = mapped_column(ForeignKey(FK_CONCEPT_ID), primary_key=True, index=True, sort_order=100)
    concept_id_2: Mapped[int] = mapped_column(ForeignKey(FK_CONCEPT_ID), primary_key=True, index=True, sort_order=200)
    relationship_id: Mapped[str] = mapped_column(
        ForeignKey(f"{VOCAB_SCHEMA}.relationship.relationship_id"), primary_key=True, index=True, sort_order=300
    )
    valid_start_date: Mapped[datetime.date] = mapped_column(Date, sort_order=400)
    valid_end_date: Mapped[datetime.date] = mapped_column(Date, sort_order=500)
    invalid_reason: Mapped[Optional[str]] = mapped_column(String(1), sort_order=600)

    @declared_attr
    def concept_1(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="ConceptRelationship.concept_id_1")

    @declared_attr
    def concept_2(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="ConceptRelationship.concept_id_2")

    @declared_attr
    def relationship(cls) -> Mapped["Relationship"]:
        return relationship("Relationship", foreign_keys="ConceptRelationship.relationship_id")


class BaseConceptSynonymCdm531:
    __tablename__ = "concept_synonym"
    __table_args__ = {"schema": VOCAB_SCHEMA}

    concept_id: Mapped[int] = mapped_column(ForeignKey(FK_CONCEPT_ID), primary_key=True, index=True, sort_order=100)
    concept_synonym_name: Mapped[str] = mapped_column(String(1000), primary_key=True, sort_order=200)
    language_concept_id: Mapped[int] = mapped_column(ForeignKey(FK_CONCEPT_ID), primary_key=True, sort_order=300)

    @declared_attr
    def concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="ConceptSynonym.concept_id")

    @declared_attr
    def language_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="ConceptSynonym.language_concept_id")


class BaseDomainCdm531:
    __tablename__ = "domain"
    __table_args__ = {"schema": VOCAB_SCHEMA}

    domain_id: Mapped[str] = mapped_column(String(20), primary_key=True, sort_order=100)
    domain_name: Mapped[str] = mapped_column(String(255), sort_order=200)
    domain_concept_id: Mapped[int] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=300)

    @declared_attr
    def domain_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="Domain.domain_concept_id")


class BaseDrugStrengthCdm531:
    __tablename__ = "drug_strength"
    __table_args__ = {"schema": VOCAB_SCHEMA}

    drug_concept_id: Mapped[int] = mapped_column(
        ForeignKey(FK_CONCEPT_ID), primary_key=True, index=True, sort_order=100
    )
    ingredient_concept_id: Mapped[int] = mapped_column(
        ForeignKey(FK_CONCEPT_ID), primary_key=True, index=True, sort_order=200
    )
    amount_value: Mapped[Optional[decimal.Decimal]] = mapped_column(Numeric, sort_order=300)
    amount_unit_concept_id: Mapped[Optional[int]] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=400)
    numerator_value: Mapped[Optional[decimal.Decimal]] = mapped_column(Numeric, sort_order=500)
    numerator_unit_concept_id: Mapped[Optional[int]] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=600)
    denominator_value: Mapped[Optional[decimal.Decimal]] = mapped_column(Numeric, sort_order=700)
    denominator_unit_concept_id: Mapped[Optional[int]] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=800)
    box_size: Mapped[Optional[int]] = mapped_column(Integer, sort_order=900)
    valid_start_date: Mapped[datetime.date] = mapped_column(Date, sort_order=1000)
    valid_end_date: Mapped[datetime.date] = mapped_column(Date, sort_order=1100)
    invalid_reason: Mapped[Optional[str]] = mapped_column(String(1), sort_order=1200)

    @declared_attr
    def amount_unit_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="DrugStrength.amount_unit_concept_id")

    @declared_attr
    def denominator_unit_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="DrugStrength.denominator_unit_concept_id")

    @declared_attr
    def drug_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="DrugStrength.drug_concept_id")

    @declared_attr
    def ingredient_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="DrugStrength.ingredient_concept_id")

    @declared_attr
    def numerator_unit_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="DrugStrength.numerator_unit_concept_id")


class BaseRelationshipCdm531:
    __tablename__ = "relationship"
    __table_args__ = {"schema": VOCAB_SCHEMA}

    relationship_id: Mapped[str] = mapped_column(String(20), primary_key=True, sort_order=100)
    relationship_name: Mapped[str] = mapped_column(String(255), sort_order=200)
    is_hierarchical: Mapped[str] = mapped_column(String(1), sort_order=300)
    defines_ancestry: Mapped[str] = mapped_column(String(1), sort_order=400)
    reverse_relationship_id: Mapped[str] = mapped_column(
        ForeignKey(f"{VOCAB_SCHEMA}.relationship.relationship_id"), sort_order=500
    )
    relationship_concept_id: Mapped[int] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=600)

    @declared_attr
    def relationship_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="Relationship.relationship_concept_id")

    @declared_attr
    def reverse_relationship(cls) -> Mapped["Relationship"]:
        return relationship("Relationship", foreign_keys="Relationship.reverse_relationship_id")


class BaseVocabularyCdm531:
    __tablename__ = "vocabulary"
    __table_args__ = {"schema": VOCAB_SCHEMA}

    vocabulary_id: Mapped[str] = mapped_column(String(20), primary_key=True, sort_order=100)
    vocabulary_name: Mapped[str] = mapped_column(String(255), sort_order=200)
    vocabulary_reference: Mapped[str] = mapped_column(String(255), sort_order=300)
    vocabulary_version: Mapped[Optional[str]] = mapped_column(String(255), sort_order=400)
    vocabulary_concept_id: Mapped[int] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=500)

    @declared_attr
    def vocabulary_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="Vocabulary.vocabulary_concept_id")


class BaseSourceToConceptMapCdm531:
    __tablename__ = "source_to_concept_map"
    __table_args__ = {"schema": CDM_SCHEMA}

    source_code: Mapped[str] = mapped_column(String(1000), primary_key=True, index=True, sort_order=100)
    source_concept_id: Mapped[int] = mapped_column(Integer, sort_order=200)
    source_vocabulary_id: Mapped[str] = mapped_column(
        ForeignKey(FK_VOCABULARY_ID), primary_key=True, index=True, sort_order=300
    )
    source_code_description: Mapped[Optional[str]] = mapped_column(String(255), sort_order=400)
    target_concept_id: Mapped[int] = mapped_column(
        ForeignKey(FK_CONCEPT_ID), primary_key=True, index=True, sort_order=500
    )
    target_vocabulary_id: Mapped[str] = mapped_column(ForeignKey(FK_VOCABULARY_ID), index=True, sort_order=600)
    valid_start_date: Mapped[datetime.date] = mapped_column(Date, sort_order=700)
    valid_end_date: Mapped[datetime.date] = mapped_column(Date, primary_key=True, sort_order=800)
    invalid_reason: Mapped[Optional[str]] = mapped_column(String(1), sort_order=900)

    @declared_attr
    def source_vocabulary(cls) -> Mapped["Vocabulary"]:
        return relationship("Vocabulary", foreign_keys="SourceToConceptMap.source_vocabulary_id")

    @declared_attr
    def target_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="SourceToConceptMap.target_concept_id")

    @declared_attr
    def target_vocabulary(cls) -> Mapped["Vocabulary"]:
        return relationship("Vocabulary", foreign_keys="SourceToConceptMap.target_vocabulary_id")


class BaseSourceToConceptMapVersionCdm531:
    __tablename__ = "source_to_concept_map_version"
    __table_args__ = {"schema": CDM_SCHEMA}

    source_vocabulary_id: Mapped[str] = mapped_column(ForeignKey(FK_VOCABULARY_ID), primary_key=True, sort_order=100)
    stcm_version: Mapped[str] = mapped_column(String(255), sort_order=200)
    last_upload_date: Mapped[datetime.datetime] = mapped_column(DateTime, default=get_current_time_utc, sort_order=300)

    @declared_attr
    def source_vocabulary(cls) -> Mapped["Vocabulary"]:
        return relationship("Vocabulary", foreign_keys="SourceToConceptMapVersion.source_vocabulary_id")



"""Module containing the placeholder schema names as used in the ORM."""

# These are the placeholder schema names as used in the ORM table
# definitions. They will be replaced with actual schema names at runtime
# by providing a schema_translate_map to your SQLAlchemy engine.
VOCAB_SCHEMA = "vocabulary_schema"
CDM_SCHEMA = "cdm_schema"

# Some often used foreign keys are added as variables for convenience.
FK_CONCEPT_ID = f"{VOCAB_SCHEMA}.concept.concept_id"
FK_VOCABULARY_ID = f"{VOCAB_SCHEMA}.vocabulary.vocabulary_id"
FK_DOMAIN_ID = f"{VOCAB_SCHEMA}.domain.domain_id"
FK_CONCEPT_CLASS_ID = f"{VOCAB_SCHEMA}.concept_class.concept_class_id"
FK_PERSON_ID = f"{CDM_SCHEMA}.person.person_id"
FK_VISIT_OCCURRENCE_ID = f"{CDM_SCHEMA}.visit_occurrence.visit_occurrence_id"
FK_VISIT_DETAIL_ID = f"{CDM_SCHEMA}.visit_detail.visit_detail_id"
FK_PROVIDER_ID = f"{CDM_SCHEMA}.provider.provider_id"
FK_CARE_SITE_ID = f"{CDM_SCHEMA}.care_site.care_site_id"
FK_LOCATION_ID = f"{CDM_SCHEMA}.location.location_id"

# This object should be used to set the naming_convention parameter of
# the SQLAlchemy MetaData object. Using it makes sure that the naming of
# constraints and indexes is deterministic and not left up to the DBMS.
# It's also needed when dropping these, as SQLAlchemy requires that
# they have a name for them to be dropped. See:
# https://docs.sqlalchemy.org/en/20/core/constraints.html#configuring-constraint-naming-conventions
NAMING_CONVENTION = {
    "ix": "ix_%(table_name)s_%(column_0_N_name)s",
    "uq": "uq_%(table_name)s_%(column_0_name)s",
    "ck": "ck_%(table_name)s_%(constraint_name)s",
    "fk": "fk_%(table_name)s_%(column_0_name)s_%(referred_table_name)s",
    "pk": "pk_%(table_name)s",
}



"""
OMOP CDM Version 5.4.

Generated from OMOP CDM release 5.4 using sqlacodegen 3.0.0rc5.
DDL from https://github.com/OHDSI/CommonDataModel/tree/v5.4



Deviations from standard model

Removed

cohort
cohort_attribute


Added

stem_table
source_to_concept_map_version


Updated

source_to_concept_map
    source_code from VARCHAR(50) --> VARCHAR(1000)
    If using a separate vocab schema, STCM is created in the CDM schema

"""

from omop_cdm.dynamic.cdm54.clinical_data import (
    BaseConditionOccurrenceCdm54,
    BaseDeathCdm54,
    BaseDeviceExposureCdm54,
    BaseDrugExposureCdm54,
    BaseFactRelationshipCdm54,
    BaseMeasurementCdm54,
    BaseNoteCdm54,
    BaseNoteNlpCdm54,
    BaseObservationCdm54,
    BaseObservationPeriodCdm54,
    BasePersonCdm54,
    BaseProcedureOccurrenceCdm54,
    BaseSpecimenCdm54,
    BaseStemTableCdm54,
    BaseVisitDetailCdm54,
    BaseVisitOccurrenceCdm54,
)
from omop_cdm.dynamic.cdm54.derived_elements import (
    BaseConditionEraCdm54,
    BaseDoseEraCdm54,
    BaseDrugEraCdm54,
    BaseEpisodeCdm54,
    BaseEpisodeEventCdm54,
)
from omop_cdm.dynamic.cdm54.health_economics import (
    BaseCostCdm54,
    BasePayerPlanPeriodCdm54,
)
from omop_cdm.dynamic.cdm54.health_system_data import (
    BaseCareSiteCdm54,
    BaseLocationCdm54,
    BaseProviderCdm54,
)
from omop_cdm.dynamic.cdm54.metadata import (
    BaseCdmSourceCdm54,
    BaseMetadataCdm54,
)
from omop_cdm.dynamic.cdm54.vocabularies.vocabularies import (
    BaseConceptAncestorCdm54,
    BaseConceptCdm54,
    BaseConceptClassCdm54,
    BaseConceptRelationshipCdm54,
    BaseConceptSynonymCdm54,
    BaseDomainCdm54,
    BaseDrugStrengthCdm54,
    BaseRelationshipCdm54,
    BaseSourceToConceptMapCdm54,
    BaseSourceToConceptMapVersionCdm54,
    BaseVocabularyCdm54,
)



"""OMOP CDM 6.0.0 derived elements tables."""

import datetime
import decimal
from typing import Optional

from sqlalchemy import BigInteger, DateTime, ForeignKey, Integer, Numeric
from sqlalchemy.orm import Mapped, declared_attr, mapped_column, relationship

from omop_cdm.constants import CDM_SCHEMA, FK_CONCEPT_ID, FK_PERSON_ID


class BaseConditionEraCdm600:
    __tablename__ = "condition_era"
    __table_args__ = {"schema": CDM_SCHEMA}

    condition_era_id: Mapped[int] = mapped_column(BigInteger, primary_key=True, sort_order=100)
    person_id: Mapped[int] = mapped_column(ForeignKey(FK_PERSON_ID), index=True, sort_order=200)
    condition_concept_id: Mapped[int] = mapped_column(ForeignKey(FK_CONCEPT_ID), index=True, sort_order=300)
    condition_era_start_datetime: Mapped[datetime.datetime] = mapped_column(DateTime, sort_order=400)
    condition_era_end_datetime: Mapped[datetime.datetime] = mapped_column(DateTime, sort_order=500)
    condition_occurrence_count: Mapped[Optional[int]] = mapped_column(Integer, sort_order=600)

    @declared_attr
    def condition_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="ConditionEra.condition_concept_id")

    @declared_attr
    def person(cls) -> Mapped["Person"]:
        return relationship("Person", foreign_keys="ConditionEra.person_id")


class BaseDoseEraCdm600:
    __tablename__ = "dose_era"
    __table_args__ = {"schema": CDM_SCHEMA}

    dose_era_id: Mapped[int] = mapped_column(BigInteger, primary_key=True, sort_order=100)
    person_id: Mapped[int] = mapped_column(ForeignKey(FK_PERSON_ID), index=True, sort_order=200)
    drug_concept_id: Mapped[int] = mapped_column(ForeignKey(FK_CONCEPT_ID), index=True, sort_order=300)
    unit_concept_id: Mapped[int] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=400)
    dose_value: Mapped[decimal.Decimal] = mapped_column(Numeric, sort_order=500)
    dose_era_start_datetime: Mapped[datetime.datetime] = mapped_column(DateTime, sort_order=600)
    dose_era_end_datetime: Mapped[datetime.datetime] = mapped_column(DateTime, sort_order=700)

    @declared_attr
    def drug_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="DoseEra.drug_concept_id")

    @declared_attr
    def person(cls) -> Mapped["Person"]:
        return relationship("Person", foreign_keys="DoseEra.person_id")

    @declared_attr
    def unit_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="DoseEra.unit_concept_id")


class BaseDrugEraCdm600:
    __tablename__ = "drug_era"
    __table_args__ = {"schema": CDM_SCHEMA}

    drug_era_id: Mapped[int] = mapped_column(BigInteger, primary_key=True, sort_order=100)
    person_id: Mapped[int] = mapped_column(ForeignKey(FK_PERSON_ID), index=True, sort_order=200)
    drug_concept_id: Mapped[int] = mapped_column(ForeignKey(FK_CONCEPT_ID), index=True, sort_order=300)
    drug_era_start_datetime: Mapped[datetime.datetime] = mapped_column(DateTime, sort_order=400)
    drug_era_end_datetime: Mapped[datetime.datetime] = mapped_column(DateTime, sort_order=500)
    drug_exposure_count: Mapped[Optional[int]] = mapped_column(Integer, sort_order=600)
    gap_days: Mapped[Optional[int]] = mapped_column(Integer, sort_order=700)

    @declared_attr
    def drug_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="DrugEra.drug_concept_id")

    @declared_attr
    def person(cls) -> Mapped["Person"]:
        return relationship("Person", foreign_keys="DrugEra.person_id")



"""OMOP CDM 5.3.1 health system data tables."""

from typing import Optional

from sqlalchemy import ForeignKey, Integer, String
from sqlalchemy.orm import Mapped, declared_attr, mapped_column, relationship

from omop_cdm.constants import CDM_SCHEMA, FK_CARE_SITE_ID, FK_CONCEPT_ID, FK_LOCATION_ID


class BaseCareSiteCdm531:
    __tablename__ = "care_site"
    __table_args__ = {"schema": CDM_SCHEMA}

    care_site_id: Mapped[int] = mapped_column(Integer, primary_key=True, sort_order=100)
    care_site_name: Mapped[Optional[str]] = mapped_column(String(255), sort_order=200)
    place_of_service_concept_id: Mapped[Optional[int]] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=300)
    location_id: Mapped[Optional[int]] = mapped_column(ForeignKey(FK_LOCATION_ID), sort_order=400)
    care_site_source_value: Mapped[Optional[str]] = mapped_column(String(50), sort_order=500)
    place_of_service_source_value: Mapped[Optional[str]] = mapped_column(String(50), sort_order=600)

    @declared_attr
    def location(cls) -> Mapped["Location"]:
        return relationship("Location")

    @declared_attr
    def place_of_service_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept")


class BaseLocationCdm531:
    __tablename__ = "location"
    __table_args__ = {"schema": CDM_SCHEMA}

    location_id: Mapped[int] = mapped_column(Integer, primary_key=True, sort_order=100)
    address_1: Mapped[Optional[str]] = mapped_column(String(50), sort_order=200)
    address_2: Mapped[Optional[str]] = mapped_column(String(50), sort_order=300)
    city: Mapped[Optional[str]] = mapped_column(String(50), sort_order=400)
    state: Mapped[Optional[str]] = mapped_column(String(2), sort_order=500)
    zip: Mapped[Optional[str]] = mapped_column(String(9), sort_order=600)
    county: Mapped[Optional[str]] = mapped_column(String(20), sort_order=700)
    location_source_value: Mapped[Optional[str]] = mapped_column(String(50), sort_order=800)


class BaseProviderCdm531:
    __tablename__ = "provider"
    __table_args__ = {"schema": CDM_SCHEMA}

    provider_id: Mapped[int] = mapped_column(Integer, primary_key=True, sort_order=100)
    provider_name: Mapped[Optional[str]] = mapped_column(String(255), sort_order=200)
    npi: Mapped[Optional[str]] = mapped_column(String(20), sort_order=300)
    dea: Mapped[Optional[str]] = mapped_column(String(20), sort_order=400)
    specialty_concept_id: Mapped[Optional[int]] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=500)
    care_site_id: Mapped[Optional[int]] = mapped_column(ForeignKey(FK_CARE_SITE_ID), sort_order=600)
    year_of_birth: Mapped[Optional[int]] = mapped_column(Integer, sort_order=700)
    gender_concept_id: Mapped[Optional[int]] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=800)
    provider_source_value: Mapped[Optional[str]] = mapped_column(String(50), sort_order=900)
    specialty_source_value: Mapped[Optional[str]] = mapped_column(String(50), sort_order=1000)
    specialty_source_concept_id: Mapped[Optional[int]] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=1100)
    gender_source_value: Mapped[Optional[str]] = mapped_column(String(50), sort_order=1200)
    gender_source_concept_id: Mapped[Optional[int]] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=1300)

    @declared_attr
    def care_site(cls) -> Mapped["CareSite"]:
        return relationship("CareSite", foreign_keys="Provider.care_site_id")

    @declared_attr
    def gender_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="Provider.gender_concept_id")

    @declared_attr
    def gender_source_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="Provider.gender_source_concept_id")

    @declared_attr
    def specialty_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="Provider.specialty_concept_id")

    @declared_attr
    def specialty_source_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="Provider.specialty_source_concept_id")



"""CDM 5.4 vocabulary tables."""

from omop_cdm.dynamic.cdm54.vocabularies.vocabularies import (
    BaseConceptAncestorCdm54,
    BaseConceptCdm54,
    BaseConceptClassCdm54,
    BaseConceptRelationshipCdm54,
    BaseConceptSynonymCdm54,
    BaseDomainCdm54,
    BaseDrugStrengthCdm54,
    BaseRelationshipCdm54,
    BaseSourceToConceptMapCdm54,
    BaseSourceToConceptMapVersionCdm54,
    BaseVocabularyCdm54,
)



"""OMOP CDM 5.4 health economics tables."""

import decimal
from datetime import datetime
from typing import Optional

from sqlalchemy import Date, ForeignKey, Integer, Numeric, String
from sqlalchemy.ext.declarative import declared_attr
from sqlalchemy.orm import Mapped, mapped_column, relationship

from omop_cdm.constants import CDM_SCHEMA, FK_CONCEPT_ID, FK_DOMAIN_ID, FK_PERSON_ID
from omop_cdm.util import record_as_str


class BasePayerPlanPeriodCdm54:
    __tablename__ = "payer_plan_period"
    __table_args__ = {"schema": CDM_SCHEMA}
    __repr__ = record_as_str

    payer_plan_period_id: Mapped[int] = mapped_column(Integer, primary_key=True, sort_order=100)
    person_id: Mapped[int] = mapped_column(ForeignKey(FK_PERSON_ID, ondelete="CASCADE"), index=True, sort_order=200)
    payer_plan_period_start_date: Mapped[datetime.date] = mapped_column(Date, sort_order=300)
    payer_plan_period_end_date: Mapped[datetime.date] = mapped_column(Date, sort_order=400)
    payer_concept_id: Mapped[Optional[int]] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=500)
    payer_source_value: Mapped[Optional[str]] = mapped_column(String(50), sort_order=600)
    payer_source_concept_id: Mapped[Optional[int]] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=700)
    plan_concept_id: Mapped[Optional[int]] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=800)
    plan_source_value: Mapped[Optional[str]] = mapped_column(String(50), sort_order=900)
    plan_source_concept_id: Mapped[Optional[int]] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=1000)
    sponsor_concept_id: Mapped[Optional[int]] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=1100)
    sponsor_source_value: Mapped[Optional[str]] = mapped_column(String(50), sort_order=1200)
    sponsor_source_concept_id: Mapped[Optional[int]] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=1300)
    family_source_value: Mapped[Optional[str]] = mapped_column(String(50), sort_order=1400)
    stop_reason_concept_id: Mapped[Optional[int]] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=1500)
    stop_reason_source_value: Mapped[Optional[str]] = mapped_column(String(50), sort_order=1600)
    stop_reason_source_concept_id: Mapped[Optional[int]] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=1700)

    @declared_attr
    def payer_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="PayerPlanPeriod.payer_concept_id")

    @declared_attr
    def payer_source_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="PayerPlanPeriod.payer_source_concept_id")

    @declared_attr
    def person(cls) -> Mapped["Person"]:
        return relationship("Person", foreign_keys="PayerPlanPeriod.person_id")

    @declared_attr
    def plan_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="PayerPlanPeriod.plan_concept_id")

    @declared_attr
    def plan_source_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="PayerPlanPeriod.plan_source_concept_id")

    @declared_attr
    def sponsor_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="PayerPlanPeriod.sponsor_concept_id")

    @declared_attr
    def sponsor_source_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="PayerPlanPeriod.sponsor_source_concept_id")

    @declared_attr
    def stop_reason_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="PayerPlanPeriod.stop_reason_concept_id")

    @declared_attr
    def stop_reason_source_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="PayerPlanPeriod.stop_reason_source_concept_id")


class BaseCostCdm54:
    __tablename__ = "cost"
    __table_args__ = {"schema": CDM_SCHEMA}
    __repr__ = record_as_str

    cost_id: Mapped[int] = mapped_column(Integer, primary_key=True, sort_order=100)
    cost_event_id: Mapped[int] = mapped_column(Integer, index=True, sort_order=200)
    cost_domain_id: Mapped[str] = mapped_column(ForeignKey(FK_DOMAIN_ID), sort_order=300)
    cost_type_concept_id: Mapped[int] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=400)
    currency_concept_id: Mapped[Optional[int]] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=500)
    total_charge: Mapped[Optional[decimal.Decimal]] = mapped_column(Numeric, sort_order=600)
    total_cost: Mapped[Optional[decimal.Decimal]] = mapped_column(Numeric, sort_order=700)
    total_paid: Mapped[Optional[decimal.Decimal]] = mapped_column(Numeric, sort_order=800)
    paid_by_payer: Mapped[Optional[decimal.Decimal]] = mapped_column(Numeric, sort_order=900)
    paid_by_patient: Mapped[Optional[decimal.Decimal]] = mapped_column(Numeric, sort_order=1000)
    paid_patient_copay: Mapped[Optional[decimal.Decimal]] = mapped_column(Numeric, sort_order=1100)
    paid_patient_coinsurance: Mapped[Optional[decimal.Decimal]] = mapped_column(Numeric, sort_order=1200)
    paid_patient_deductible: Mapped[Optional[decimal.Decimal]] = mapped_column(Numeric, sort_order=1300)
    paid_by_primary: Mapped[Optional[decimal.Decimal]] = mapped_column(Numeric, sort_order=1400)
    paid_ingredient_cost: Mapped[Optional[decimal.Decimal]] = mapped_column(Numeric, sort_order=1500)
    paid_dispensing_fee: Mapped[Optional[decimal.Decimal]] = mapped_column(Numeric, sort_order=1600)
    payer_plan_period_id: Mapped[Optional[int]] = mapped_column(
        ForeignKey(f"{CDM_SCHEMA}.payer_plan_period.payer_plan_period_id"), sort_order=1700
    )
    amount_allowed: Mapped[Optional[decimal.Decimal]] = mapped_column(Numeric, sort_order=1800)
    revenue_code_concept_id: Mapped[Optional[int]] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=1900)
    revenue_code_source_value: Mapped[Optional[str]] = mapped_column(String(50), sort_order=2000)
    drg_concept_id: Mapped[Optional[int]] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=2100)
    drg_source_value: Mapped[Optional[str]] = mapped_column(String(3), sort_order=2200)

    @declared_attr
    def cost_domain(cls) -> Mapped["Domain"]:
        return relationship("Domain", foreign_keys="Cost.cost_domain_id")

    @declared_attr
    def cost_type_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="Cost.cost_type_concept_id")

    @declared_attr
    def currency_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="Cost.currency_concept_id")

    @declared_attr
    def drg_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="Cost.drg_concept_id")

    @declared_attr
    def revenue_code_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="Cost.revenue_code_concept_id")



"""OMOP CDM legacy tables."""

import datetime
import decimal
from typing import Optional

from sqlalchemy import Date, ForeignKey, Integer, Numeric, String, Text
from sqlalchemy.orm import Mapped, declared_attr, mapped_column, relationship

from omop_cdm.constants import CDM_SCHEMA, FK_CONCEPT_ID

FK_COHORT_DEFINITION_ID = f"{CDM_SCHEMA}.cohort_definition.cohort_definition_id"


class BaseCohort:
    __tablename__ = "cohort"
    __table_args__ = {"schema": CDM_SCHEMA}

    cohort_definition_id: Mapped[int] = mapped_column(
        ForeignKey(FK_COHORT_DEFINITION_ID), primary_key=True, index=True, sort_order=100
    )
    subject_id: Mapped[int] = mapped_column(Integer, primary_key=True, index=True, sort_order=200)
    cohort_start_date: Mapped[Date] = mapped_column(Date, primary_key=True, sort_order=300)
    cohort_end_date: Mapped[Date] = mapped_column(Date, primary_key=True, sort_order=400)

    @declared_attr
    def cohort_definition(cls) -> Mapped["CohortDefinition"]:
        return relationship("CohortDefinition")


class BaseCohortDefinition:
    __tablename__ = "cohort_definition"
    __table_args__ = {"schema": CDM_SCHEMA}

    cohort_definition_id: Mapped[Integer] = mapped_column(Integer, primary_key=True, index=True, sort_order=100)
    cohort_definition_name: Mapped[str] = mapped_column(String(255), sort_order=200)
    cohort_definition_description: Mapped[Optional[str]] = mapped_column(Text, sort_order=300)
    definition_type_concept_id: Mapped[int] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=400)
    cohort_definition_syntax: Mapped[Optional[str]] = mapped_column(Text, sort_order=500)
    subject_concept_id: Mapped[int] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=600)
    cohort_initiation_date: Mapped[Optional[datetime.date]] = mapped_column(Date, sort_order=700)

    @declared_attr
    def definition_type_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="CohortDefinition.definition_type_concept_id")

    @declared_attr
    def subject_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="CohortDefinition.subject_concept_id")


class BaseCohortAttribute:
    __tablename__ = "cohort_attribute"
    __table_args__ = {"schema": CDM_SCHEMA}

    cohort_definition_id: Mapped[int] = mapped_column(
        ForeignKey(FK_COHORT_DEFINITION_ID), primary_key=True, index=True, sort_order=100
    )
    subject_id: Mapped[int] = mapped_column(Integer, primary_key=True, sort_order=200)
    cohort_start_date: Mapped[datetime.date] = mapped_column(Date, primary_key=True, sort_order=300)
    cohort_end_date: Mapped[datetime.date] = mapped_column(Date, primary_key=True, sort_order=400)
    attribute_definition_id: Mapped[int] = mapped_column(
        ForeignKey(f"{CDM_SCHEMA}.attribute_definition.attribute_definition_id"),
        primary_key=True,
        index=True,
        sort_order=500,
    )
    value_as_number: Mapped[Optional[decimal.Decimal]] = mapped_column(Numeric, sort_order=600)
    value_as_concept_id: Mapped[Optional[int]] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=700)

    @declared_attr
    def cohort_definition(cls) -> Mapped["CohortDefinition"]:
        return relationship("CohortDefinition")

    @declared_attr
    def attribute_definition(cls) -> Mapped["AttributeDefinition"]:
        return relationship("AttributeDefinition")

    @declared_attr
    def value_as_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept")


class BaseAttributeDefinition:
    __tablename__ = "attribute_definition"
    __table_args__ = {"schema": CDM_SCHEMA}

    attribute_definition_id: Mapped[int] = mapped_column(Integer, primary_key=True, index=True, sort_order=100)
    attribute_name: Mapped[str] = mapped_column(String(255), sort_order=200)
    attribute_description: Mapped[Optional[str]] = mapped_column(Text, sort_order=300)
    attribute_type_concept_id: Mapped[int] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=400)
    attribute_syntax: Mapped[Optional[str]] = mapped_column(Text, sort_order=500)

    @declared_attr
    def attribute_type_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept")



import datetime

from sqlalchemy.orm import DeclarativeBase


def record_as_str(record: DeclarativeBase) -> str:
    col_name_values = {
        cname: str(getattr(record, cname))
        for cname in record.__class__.__table__.columns.keys()  # noqa: SIM118
    }
    return (
        f"{record.__class__.__name__}("
        f"""{
            ", ".join(
                f"{c_name}={c_value}" for c_name, c_value in col_name_values.items()
            )
        })"""
    )


def get_current_time_utc() -> datetime.datetime:
    return datetime.datetime.now(tz=datetime.timezone.utc)
